class LMS_Color:

    def __init__(
        self, red: int = None, green: int = None, blue: int = None, alpha: int = None
    ):
        self.name: str = None
        self.red = red
        self.green = green
        self.blue = blue
        self.alpha = alpha
